@@ -3,7 +3,7 @@
 //
 //  This file is part of Linterna Mágica
 //
-//  Copyright (C) 2011, 2012  Ivaylo Valkov <ivaylo@e-valkov.org>
+//  Copyright (C) 2011, 2012, 2014 Ivaylo Valkov <ivaylo@e-valkov.org>
 //
 //  The JavaScript code in this page (or file) is free software: you
 //  can redistribute it and/or modify it under the terms of the GNU
@@ -22,7 +22,7 @@
 //  @licend The above is the entire license notice for the JavaScript
 //  code in this page (or file).
 //
-// @source http://linterna-magica.nongnu.org
+// @source http://linternamagica.org
 
 // END OF LICENSE HEADER
 
@@ -53,7 +53,6 @@ function(linterna_magica_id)
     for (var i=0, l=object_list.length; i<l; i++)
     {
 	var o = object_list[i];
-
 	if (o.linterna_magica_id != undefined &&
 	    o.linterna_magica_id == linterna_magica_id)
 	{
@@ -112,6 +111,18 @@ LinternaMagica.prototype.object_has_css_class = function (element, className)
     return false;
 }
 
+// Add a class to the classes of an element
+LinternaMagica.prototype.add_css_class = function(element, class_name)
+{
+    if (!this.object_has_css_class(element, class_name))
+    {
+	var c = element.getAttribute("class");
+	element.setAttribute("class", c+" "+class_name);
+    }
+
+    return element.getAttribute("class");
+}
+
 // Get the video object with id
 // "linterna-magica-video-object-"+linterna_magica_id from DOM
 LinternaMagica.prototype.get_video_object = function(linterna_magica_id)
@@ -125,43 +136,76 @@ LinternaMagica.prototype.get_video_object = function(linterna_magica_id)
 }
 
 // Show the video object and the interface
-LinternaMagica.prototype.show_lm_video = function(linterna_magica_id)
+LinternaMagica.prototype.show_lm_interface = function(linterna_magica_id)
 {
-    var lm = this.get_video_object(linterna_magica_id).parentNode;
-
-    if (!lm)
-    {
-	return null;
-    }
-
-    lm.style.removeProperty("display");
+    var lm_interface = document.getElementById("linterna-magica-"+
+					       linterna_magica_id);
+    this.__show_lm(lm_interface);
 }
 
 // Hide the video object and the interface
-LinternaMagica.prototype.hide_lm_video = function(linterna_magica_id)
+LinternaMagica.prototype.hide_lm_interface = function(linterna_magica_id)
 {
-    var lm = this.get_video_object(linterna_magica_id).parentNode;
+    var lm_interface = document.getElementById("linterna-magica-"+
+					       linterna_magica_id);
+    this.__hide_lm(lm_interface, linterna_magica_id);
+}
 
-    if (!lm)
+// Hide the video object and the interface
+LinternaMagica.prototype.hide_lm_video_object = function(linterna_magica_id)
+{
+    var video_object = document.getElementById("linterna-magica-"+
+					       "video-object-wrapper-"+
+					       linterna_magica_id);
+    this.__hide_lm(video_object, linterna_magica_id);
+}
+
+// Show the video object
+LinternaMagica.prototype.show_lm_video_object = function(linterna_magica_id)
+{
+    var video_object = document.getElementById("linterna-magica-"+
+					       "video-object-wrapper-"+
+					       linterna_magica_id);
+    this.__show_lm(video_object);
+}
+
+LinternaMagica.prototype.__show_lm = function(element)
+{
+    if (!element)
     {
 	return null;
     }
 
-    lm.style.setProperty("display", "none", "important");
+    element.style.removeProperty("display");
 }
 
+LinternaMagica.prototype.__hide_lm = function(element, linterna_magica_id)
+{
+    if (!element)
+    {
+	return null;
+    }
+
+    var self = this;
+    this.player.stop.apply(self, [linterna_magica_id]);
+
+    element.style.setProperty("display", "none", "important");
+}
+
+
 // Show the flash video object
 LinternaMagica.prototype.show_flash_video_object =
-function(linterna_magica_id,parent)
+function(flash_object)
 {
-    var flash_object = 
-	this.get_flash_video_object(linterna_magica_id,parent);
-
     if (!flash_object)
     {
 	return null;
     }
 
+
+    // See https://savannah.nongnu.org/bugs/?36888
+    flash_object = this.force_flash_video_object_start(flash_object);
+
     flash_object.style.removeProperty("display");
 
     return flash_object;
@@ -169,21 +213,141 @@ function(linterna_magica_id,parent)
 
 // Hide the flash video object
 LinternaMagica.prototype.hide_flash_video_object =
-function(linterna_magica_id, parent)
+function(flash_object)
 {
-    var flash_object =
-	this.get_flash_video_object(linterna_magica_id, parent);
-
     if (!flash_object)
     {
 	return null;
     }
 
+    // Force the playback to stop. Setting the object on the
+    // background and changing its type alone does not work. 
+    // See https://savannah.nongnu.org/bugs/?36888
+    flash_object = this.force_flash_video_object_stop(flash_object);
+
     flash_object.style.setProperty("display", "none", "important");
 
     return flash_object;
 }
 
+LinternaMagica.prototype.force_flash_video_object_stop =
+function(flash_object)
+{
+    var clone = this.force_flash_video_object_src(flash_object, 
+						     "x-fake-flash");
+    clone = 
+	this.force_flash_video_object_type(clone,
+				       "x-fake/x-flash-stopped");
+
+    clone = this.force_flash_video_object_injection(flash_object, clone);
+
+    return clone;
+}
+
+LinternaMagica.prototype.force_flash_video_object_start =
+function(flash_object)
+{
+    var clone = this.force_flash_video_object_src(flash_object, "swf");
+
+    clone = 
+	this.force_flash_video_object_type(clone,
+				       "application/x-shockwave-flash");
+
+    clone = this.force_flash_video_object_injection(flash_object, clone);
+
+    return clone;
+}
+
+// Add/remove swf from the flash object src/data attribute. Used when
+// hiding the flash player to stop its playback so there is no dual
+// playback as described in bug #36888.
+// See https://savannah.nongnu.org/bugs/?36888
+LinternaMagica.prototype.force_flash_video_object_src =
+function(flash_object, src)
+{
+    if (!flash_object || !src)
+    {
+	return null;
+    }
+
+    var old_src = null;
+
+    if (/swf/i.test(src))
+    {
+	old_src = "x-fake-flash";
+    }
+    else if (/x-fake-flash/i.test(src))
+    {
+	old_src = "swf";
+    }
+    else
+    {
+	return flash_object;
+    }
+
+    var src_attribute = 
+	/object/i.test(flash_object.localName) ? "data" : "src";
+
+    var full_src = flash_object.getAttribute(src_attribute);
+
+    if (!full_src)
+    {
+	return flash_object;
+    }
+
+    full_src = full_src.replace('.'+old_src, '.'+src);
+
+    var clone = flash_object.cloneNode(true);
+    clone.linterna_magica_id = flash_object.linterna_magica_id;
+    
+    clone.setAttribute(src_attribute, full_src);
+
+    return clone;
+}
+
+LinternaMagica.prototype.force_flash_video_object_injection =
+function(flash_object, clone)
+{
+    var sibling = flash_object.nextSibling ? 
+	flash_object.nextSibling : null;
+
+    var parent = flash_object.parentNode;
+
+    flash_object.parentNode.removeChild(flash_object);
+ 
+    if (sibling)
+    {
+	parent.insertBefore(clone, sibling);
+    }
+    else
+    {
+	parent.appendChild(clone);
+    }
+
+    return clone;
+}
+
+// Force the flash object to another type. Used when hiding the flash
+// player to stop its playback so there is no dual playback as
+// described in bug #36888.
+// See https://savannah.nongnu.org/bugs/?36888
+LinternaMagica.prototype.force_flash_video_object_type =
+function(flash_object, type)
+{
+    if (!flash_object || !type)
+    {
+	return null;
+    }
+  
+    var clone = flash_object.cloneNode(true);
+    clone.linterna_magica_id = flash_object.linterna_magica_id;
+    
+    flash_object = clone;
+    flash_object.setAttribute("type", type);
+   
+    return flash_object;
+}
+
 // Check all video objects to find if one is started.
 LinternaMagica.prototype.find_started_clip = function()
 {
@@ -287,7 +451,7 @@ function(parent)
 // Hide the HTML5 player wrapper found in the parent element.
 LinternaMagica.prototype.hide_site_html5_player =
 function(parent)
-{    
+{
     var html5_player =
 	this.find_site_html5_player_wrapper(parent);
 
